﻿using Microsoft.EntityFrameworkCore;
using ProRestaurant.Containers;
using ProRestaurant.Models;
using ProRestaurant.Models.Accounts;
using ProRestaurant.Models.Misc;
using ProRestaurant.Models.Restaurants;
using System.Linq;

namespace ProRestaurant.DBContexts
{
    public class DBContext : DbContext
    {
        private string connectionString = "";
        private bool UseSQL = false;
        public DBContext(DbContextOptions<DBContext> options) 
            : base(options)
        {            
        }

        #region Accounts
        public DbSet<User> Users { get; set; }
        public DbSet<DriverDetail> DriverDetails { get; set; }
        public DbSet<UserAddress> UserAddresses { get; set; }
        #endregion

        #region Restaurants
        public DbSet<Restaurant> Restaurants { get; set; }
        public DbSet<TradingHours> TradingHours { get; set; }
        public DbSet<MenuCategory> MenuCategories { get; set; }
        public DbSet<MenuItem> MenuItems { get; set; }
        public DbSet<MenuOption> MenuOptions { get; set; }
        public DbSet<MenuOptionItem> MenuOptionItems { get; set; }
        public DbSet<RestaurantUser> RestaurantUsers { get; set; }
        public DbSet<RestaurantCategory> RestaurantCategories { get; set; }
        #endregion

        #region Misc
        public DbSet<Locations> Locations { get; set; }
        #endregion

        public override int SaveChanges()
        {            
            foreach (var item in ChangeTracker
                                    .Entries()
                                    .Where(x => x.State == EntityState.Modified || x.State == EntityState.Added)
                                    .Select(x => x.Entity)
                                    .ToList())
            {
                if (item is BaseObject)
                {                    
                    (item as BaseObject).UpdateModified(string.Empty);
                }
            }            
            UpdateSoftDeleteStatuses();

            return base.SaveChanges();
        }

        private void UpdateSoftDeleteStatuses()
        {            
            foreach (var entry in ChangeTracker.Entries())
            {
                var props = (from p in entry.CurrentValues.Properties
                             select p.Name).ToList();

                if (props.Contains("IsDeleted"))
                {
                    switch (entry.State)
                    {
                        case EntityState.Added:
                            entry.CurrentValues["IsDeleted"] = false;
                            break;
                        case EntityState.Deleted:
                            entry.State = EntityState.Modified;
                            entry.CurrentValues["IsDeleted"] = true;
                            break;
                    }
                }
            }
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {            
            modelBuilder.Entity<User>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);
            modelBuilder.Entity<UserAddress>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);
            modelBuilder.Entity<DriverDetail>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);
            modelBuilder.Entity<Restaurant>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);
            modelBuilder.Entity<TradingHours>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);
            modelBuilder.Entity<MenuCategory>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);
            modelBuilder.Entity<MenuItem>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);
            modelBuilder.Entity<MenuOption>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);
            modelBuilder.Entity<MenuOptionItem>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);
            modelBuilder.Entity<RestaurantUser>().HasQueryFilter(m => EF.Property<bool>(m, "IsDeleted") == false);            
        }
    }
}
